% script file advection2d.m
% solves the linear advection equation
% with constant speed in 2 dimensions
% 
% last changes: 30.06.03 by Vita
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc; clear all; close all;

% load the data files
tri = load('square.tri'); % triangles
nod = load('square.nod'); % nodes

a = [0.5 0.5];

% number of triangles & nodes
n.tri = length(tri); n.nod = length(nod);

% initial data: given at the nodes
u = zeros(n.nod,1);
% put some discontinuity
mid = find(((nod(:,1)+0.25).^2+(nod(:,2)+0.25).^2)<0.05);
u(mid) = 1;


Tmax = 1;
tau = 0.05;

% compute the areas of the median cells
% first, compute areas of the triangles
disp('compute areas of triangles')
tri_area = triangle_area(nod,tri);
% then, compute areas of the median cells
disp('compute areas of the median cells')
nod_area = median_cell_area(nod,tri,tri_area);

% the initial discontinuity:
psi = linspace(0,2*pi);
xi = sqrt(0.05)*sin(psi)-0.25;
yi = sqrt(0.05)*cos(psi)-0.25;


% plot the initiaL data
figure
P=trimesh(tri,nod(:,1),nod(:,2),u);
caxis([0,1])
set(P,'marker','.')
set(P,'markersize',25)
set(P,'facecolor','none')
view(2); colorbar;
title(['t = 0']);
hold on
P=plot(xi,yi,'r');
set(P,'linewidth',2)
drawnow


disp('iteration')
t = 0;
while t<Tmax
  t = t + tau;
  
  % compute the update at each node
  Update = zeros(n.nod,1);
  
  for ctr = 1:n.tri
    % select coordinates of each node
    x1 = nod(tri(ctr,1),1); y1 = nod(tri(ctr,1),2);  
    x2 = nod(tri(ctr,2),1); y2 = nod(tri(ctr,2),2);  
    x3 = nod(tri(ctr,3),1); y3 = nod(tri(ctr,3),2); 
    
    % normals:
    nor1 = [y2-y3; x3-x2];
    nor2 = [y3-y1; x1-x3];
    nor3 = [y1-y2; x2-x1];
    
    % check if the points are positively oriented
    % for that, check if vectors for edges
    % 1 and 2 are positively oriented: use determinant condition
    % a and b are positively oriented if
    % |a1 b1|
    % |a2 b2| > 0
    
    D = (x3-x2)*(y1-y3)-(y3-y2)*(x1-x3);  
    if D<0 
      % change the orientation
      disp('directions changed')
      nor1(ctr,:) = -nor1(ctr,:);
      nor2(ctr,:) = -nor2(ctr,:);    
      nor3(ctr,:) = -nor3(ctr,:);  
    end 
   
    % compute k's
    k1 = 0.5*a*nor1; k2 = 0.5*a*nor2; k3 = 0.5*a*nor3;
    
    % compute alpha's
    alpha_denom = max(0,k1) + max(0,k2) + max(0,k3);
    alpha1 = max(0,k1)/alpha_denom;
    alpha2 = max(0,k2)/alpha_denom;    
    alpha3 = max(0,k3)/alpha_denom;
    
    % compute the fluctuation
    %keyboard
    Phi = -(u(tri(ctr,1))*k1 + u(tri(ctr,2))*k2 + u(tri(ctr,3))*k3);
    
    Update(tri(ctr,1)) = Update(tri(ctr,1))+...
	tau/nod_area(tri(ctr,1))*alpha1*Phi;
    Update(tri(ctr,2)) = Update(tri(ctr,2))+...
	tau/nod_area(tri(ctr,2))*alpha2*Phi;
    Update(tri(ctr,3)) = Update(tri(ctr,3))+...
	tau/nod_area(tri(ctr,3))*alpha3*Phi;
  end
  
  u = u + Update;
  
  % visualize the result
  clf
  P=trimesh(tri,nod(:,1),nod(:,2),u);
  caxis([0,1])
  set(P,'marker','.')
  set(P,'markersize',25)
  set(P,'facecolor','none')
  view(2); colorbar;
  axis([-0.5 0.5 -0.5 0.5]);
  title(['t = ',num2str(t)]);
  hold on
  % plot the exact discontinuity position
  P=plot(xi+a(1)*t,yi+a(2)*t,'r');
  set(P,'linewidth',2)
  drawnow
  
  
end
